c.. this program calculates the solution to the following
c   pde using explicit forward difference.
c
c
c    dc      d^2 c        dc
c   ---- = D -----   - V -----  -kc
c    dt      dx^2         dx
c..
c   Dimension the arrays
      dimension c(2,101),x(101),y(101)
      integer*4 w(1)
      logical cin
      character*1 ians
      character*80 xtitle,ytitle
c..   open the console on units 5 and 6
      open(unit=5,file='con')
      open(unit=6,file='con')
c.. get the input output information
      call opclse(nr,nw)
c..   get the boundry points
      if(nr.eq.5) then
         write(6,1000)
1000     format(' Enter the inlet boundry value ')
      endif
      read(nr,*) bconc
      if(nr.eq.5) then
         write(6,1010)
1010     format(' Enter the intial condition ')
      endif
      read(nr,*) cinit
10    if(nr.eq.5) then
         write(6,1020)
1020     format(' Enter the number of grid points',/,
     1   ' The number must be less than 100  ')
      endif
      read(nr,*) ir
      if(ir.gt.100) then
        if(nr.eq.5) then
           goto 10
        else
           write(6,*) 'The number of grid points exceeds 100'
           stop
        endif
      endif
c.. set the initial conditions
      do 20 i=2,ir
20    c(1,i)=cinit
      c(1,1)=bconc
c.. ask for the time increment
      if(nr.eq.5) then
         write(6,*) 'Enter the time increment (delta t)  '
      endif
      read(nr,*) dt
      dt2=dt/2
c.. ask for the final time
      if(nr.eq.5) then
         write(6,*) 'Enter the finish time (time max)  '
      endif
      read(nr,*) fintim
c.. ask for the reaction  rate
      if(nr.eq.5) then
        write(6,*) 'Enter the reaction rate constant   '
      endif
      read(nr,*) rk
c.. read the dispersion coefficient
      if(nr.eq.5) then
        write(6,*) 'Enter the dispersion coefficient   '
      endif
      read(nr,*) disp
c.. read the velocity
      if(nr.eq.5) then
        write(6,*) 'Enter the velocity  '
      endif
      read(nr,*) vel
c..   calculate dx
      dx=1./(ir-1)
c.. calculate the required constants
      dtdx2=disp*dt/(dx**2)
      if(dtdx2.gt.0.5) then
        write(6,1025)
1025    format(' Warning the problem is unstable ')
      endif
      dtdxt=vel*dt/(2.*dx)
c.. set the loop counters and such to zero
      time=0.
      ptime=0.
      iplcnt=0
      iter=0
      ipcont=0
      ir1=ir-1
c.. find out how often i should print
      if(nr.eq.5) then
        write(6,*) 'Enter the time printing increment   '
      endif
      read(nr,*) dprint
      if(dprint.lt.dt) dprint=dt
c.. find out how often i should plot. Skip if I am printing to
c   the screen
      if(nw.ne.6) then
        if(nr.eq.5) then
          write(6,*) 'Enter the time plotting increment   '
        endif
        read(nr,*) dplot
        if(dplot.lt.dt) dplot=dt
      else
        dplot=2*fintim
      endif
c.. set pltime to a big number if dplot is greater than fintime.
c   this prevents any calls to gks unless you are really going to
c   plot
      if(dplot.gt.fintim) pltime=2.*fintim
      if(nr.eq.5) then
        write(6,1030)
1030    format(' Do you want me to restrict the spatial ',
     1' printing (y or n)  ')
      endif
      if(cin(nr)) then
        idelta=ir/10
        write(nw,*) ' +++++++++ RESTRICTED PRINTING ++++++++ '
        write(nw,*) ' Warning: the grid points (both time and space)'
        write(nw,*) ' may not print at even intervals.'
        write(nw,*) ' '
      else
        idelta=1
      endif
c.. Calculate the Peclet number
      if(vel.gt.0) then
	peclet=vel*dx/disp
      else
        peclet=-1.
      endif
c.. Write out a summary for record keeping
      write(nw,1040) bconc,cinit,dx,ir,dt,fintim,rk,disp,vel,dprint,
     1dplot,peclet
1040  format(
     1' Inlet Boundry Concentration=',f7.2,/
     2' Initial Concentration      =',f7.2,/
     3' Distance Increment (dx)    =',f7.3,/
     4' Number of Grid Points (ir) =',4x,i3,/
     5' Time Increment (dt)        =',f7.4,/
     6' Finish Time                =',f7.1,/
     7' Reaction Rate              =',f7.3,/
     8' Dispersion Coeffecient     =',f10.7,/
     9' Velocity                   =',f7.4,/
     A' Print Interval             =',f7.4,/
     B' Plotting Increment         =',f7.4,/
     C' Peclet Number              =',f7.3)
c..  create an x and y array to for plotting
      do 29 i=1,ir
      x(i)=(i-1)*dx
29    y(i)=c(1,i)
c.. call cplot to initialize the 
      if(dplot.le.fintim) then
c       xtitle='     Distance       '
c       ytitle='   Concentration    '
c       call cplot (9.,7.5,ir,-1,x,y,5,5,w,xtitle,80,ytitle,80)
      endif
c.. now finally begin the calculations
c     loop point
30    continue
      iter=iter+1
      time=dt*float(iter)
      ix=imod2(iter)
      iy=imod2(iter+1)
      do 40 i=2,ir1
      im1=i-1
      ip1=i+1
40    c(iy,i)=dtdx2*(c(ix,ip1)-2.*c(ix,i)+c(ix,im1)) -dtdxt*
     1       (c(ix,ip1)-c(ix,im1))-dt*rk*c(ix,i)+c(ix,i) 
c..   Now do the entrance boundry
      c(iy,1)=bconc
c..   Now for the exit (zero flux)
      c(iy,ir)=dtdx2*(2.*c(ix,ir1)-2.*c(ix,ir))-dt*rk*c(ix,ir)+c(ix,ir)
c..   check to see if it's time to print
      if(time.ge.(ptime-dt2)) then
        ipcont=ipcont+1
        ptime=ipcont*dprint
        write(nw,1100) time,iter
1100    format(///,' Time=',g10.3,5x,' Number of time steps=',i6,/)
        write(nw,1110) (c(iy,j), j=1,ir,idelta)
1110    format(5(2x,g12.5))
        iwrite=1
        if(nw.eq.6) then
          write(6,*) 'Enter a Character to Continue '
          read(5,1120) ians
1120      format(1a1)
        endif
      else
        iwrite=0
      endif
c.. check to see if its time to plot
      if(time.ge.(pltime-dt2)) then
        iplcnt=iplcnt+1
        pltime=iplcnt*dplot
c.. insert the values into an array for plotting. Avoid using
c   equilivance statements
        do 50 i=1,ir
50      y(i)=c(iy,i)
c       call cplot (9.,7.5,ir,1,x,y,0,0,w,xtitle,20,ytitle,20)
        iplot=1
      else
        iplot=0
      endif
      if(time.ge.fintim) then
c.. stoping
c.. check to see that we ended on a plot or print
        if(iwrite.eq.0) then
          write(nw,1100) time,iter
          write(nw,1110) (c(iy,j), j=1,ir,idelta)
        endif
        if(iplot.eq.0.and.dplot.le.fintim) then
          do 60 i=1,ir
60        y(i)=c(iy,i)
c         call cplot(9.,7.5,ir,1,x,y,0,0,w,xtitle,20,ytitle,20)
        endif 
c.. clear the plot
        if(dplot.le.fintim) then
c         call cplot(9.,7.5,ir,5,x,y,0,0,w,xtitle,20,ytitle,20)
        endif
        write(nw,*) ' Stopping for final time'
        stop
      else 
        goto 30
      endif
      end
